
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Generate action for the whole request

FormSubmitAction, LinkAction, OpenAction, will be generated in the embedded
context of RequestAndCheckAction or RequestWithFailureAction
so instead of they generate, their generate_set and generate_submit
is called in two parts.

Plugins are generated together with the formcheck.
'''

from base import Action
from form_check import FormCheck
import traceback
from Products.mechagen.plugins import get_plugins, get_beautifulsoup_class

class RequestAndCheckAction(Action):
    '''Check the equality of the form vars for all the forms
    
    Includes generation of the actual submit action.
    '''

    realaction = True

    def __init__(self, embedded, data):
        self.embedded = embedded
        # Compare values in the data
        self.forms = []
        for nr, b_form in enumerate(data.response_forms):
            self.forms.append(FormCheck(data.response_forms, nr))
        # Collect plugins
        self.active_plugins = []
        soup_class = get_beautifulsoup_class()
        if soup_class is not None:
            soup = soup_class(data.response_body)   # or data.wrapped_response?
            for plugin_class in get_plugins():
                plugin_instance = plugin_class(soup)
                if plugin_instance.active():
                    self.active_plugins.append(plugin_instance)
            
    def generate(self, writer):
        # first, generate embedded action
        self.generate_header(writer)
        self.embedded.generate_set(writer)
        self.embedded.generate_submit(writer)
        self.generate_footer(writer)
        # then, generate the plugin checks
        if self.active_plugins:
            self.generate_header(writer, 'Auto-generated plugin check')
            writer.write('soup = PluginSoup(page)')
            writer.newline()
            for plugin in self.active_plugins:
                plugin.generate(writer)
            self.generate_footer(writer)
        # then, generate the form checks
        self.generate_header(writer, 'Auto-generated form check')
        for form in self.forms:
            form.generate(writer)
        self.generate_footer(writer)
            
class RequestWithFailureAction(Action):
    '''Submit action with failure.

    Actual submit action is embedded.
    '''

    realaction = True

    def __init__(self, embedded, publish_exc, response_status):
        self.embedded = embedded
        self.publish_exc = publish_exc
        self.response_status = response_status
            
    def generate(self, writer):
        self.generate_header(writer)
        # generate embedded action - submit part
        self.embedded.generate_set(writer)
        # start protected section
        writer.write('try:')
        writer.newline()
        writer.indenton()
        # generate embedded action - submit part
        self.embedded.generate_submit(writer)
        #
        writer.indentoff()
        writer.write('except:')
        writer.newline()
        writer.indenton()
        writer.write('t, v, tb = sys.exc_info()')
        writer.newline()
        if self.response_status is not None:
            writer.write('# Exception=%s, Response status=%s' % (self.publish_exc[0], self.response_status))
            writer.newline()
            writer.write('self.assert_(t is HTTPError, "This request should have generated an error, but in this case it raised exception %s." % (v, ))')
            writer.newline()
            status_major = self.response_status / 100
            writer.write('self.assert_(t is HTTPError and v.code / 100 == %i, "This request should have generated an %i0x error, but in this case it finished with status %%s." %% (v.code, ))' % (status_major, status_major))
            writer.newline()
            print_traceback = False
        else:
            # Unknown response status, this is perhaps ImportError or SystemExit from publish.
            writer.write('# Exception=%s' % (self.publish_exc[0], ))
            writer.newline()
            writer.write('self.assert_(t is HTTPError, "This request should have generated an error, but in this case it raised exception %s." % (v, ))')
            writer.newline()
            print_traceback = True
        if print_traceback:
            # print the full exception traceback
            writer.write('#')
            writer.newline()
            t, v, tb = self.publish_exc
            # v is the full html error - ignore it here
            v = '*** unknown exception text ***'
            for line in ''.join(traceback.format_exception(t, v, tb, limit=20)).splitlines():
                writer.write('# ' + line)
                writer.newline()
            writer.write('#')
            writer.newline()
        writer.indentoff()
        # else branch should catch if there was no error
        writer.write('else:')
        writer.newline()
        writer.indenton()
        writer.write('self.assert_(False, "This request should have generated an error, but in this case it finished OK.")')
        writer.newline()
        writer.indentoff()
        self.generate_footer(writer)
