
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Runtime support for mechagen generated code

The usage of this is optional and can be avoided with options at
test generation time. Default is using this code.

Code that wants to use extended features, must use
the ExtendedBrowser class defined in this module.
'''

__all__ = ('ExtendedBrowser', )

import ClientForm, mechanize
isstringlike = ClientForm.isstringlike

# --
# Extended HTMLForm
# --

# XXX Since ClientForms does not make HTMLForm pluggable, we
# apply a really miserable solution: We monkey-patch HTMLForm.

class HTMLForm_patches:
   
    def find_controls(self,
                     name=None, type=None, kind=None, id=None,
                     predicate=None,
                     label=None):
        """Locate and return a list of specific controls within the form.

        Parameters are like with find_control, but

        - there is no "nr"
        - if no elements are found, no exception is raised but 
          an empty list is returned.
        """
        if ((name is None) and (type is None) and (kind is None) and
            (id is None) and (label is None) and (predicate is None)):
            raise ValueError(
                "at least one argument must be supplied to specify control")
        results, _p_results = self._find_controls(name, type, kind, id, label, predicate)
        return results

    def find_zope_list_control(self, name):
        '''Encapsulates a zope :list control by name

        name should be without the ":list" in the end.
        '''
        import re
        regex = re.compile(re.escape(name) + r':list')
        
        def predicate(control):
            if control.name is None:
                return None
            else:
                return regex.match(control.name)
            
        results, p_results = self._find_controls(name=None, type=None, kind=None, id=None, 
            label=None, predicate=predicate)
        
        return results

    def find_zope_record_control(self, name):
        '''Encapsulates a zope :record control by name

        name should be without the ".<field>:record" in the end.
        '''
        import re
        regex = re.compile(re.escape(name) + r'\.([^:]+).*:record$')
        
        def predicate(control):
            if control.name is None:
                return None
            else:
                return regex.match(control.name)
        
        results, p_results = self._find_controls(name=None, type=None, kind=None, id=None, 
            label=None, predicate=predicate)
        d = {}
        for i in range(len(results)):
            key = p_results[i].group(1)
            d[key] = results[i]
        return d

    def find_zope_records_control(self, name):
        '''Encapsulates a zope :records control by name

        name should be without the ".<field>:records" in the end.
        '''
        import re
        regex = re.compile(re.escape(name) + r'\.([^:]+).*:records$')
        
        def predicate(control):
            if control.name is None:
                return None
            else:
                return regex.match(control.name)
        
        results, p_results = self._find_controls(name=None, type=None, kind=None, id=None, 
            label=None, predicate=predicate)
        dl = [{}]
        for i in range(len(results)):
            key = p_results[i].group(1)
            if key in dl[-1]:
                # start of new row
                dl.append({})
            dl[-1][key] = results[i]
        # remove marker if empty
        if dl[-1] == {}:
            dl.pop()
        return dl

    def _find_controls(self, name, type, kind, id, label, predicate):
        '''Returns a tuple with info about all the matching controls.

        First item is a list of all matching controls.

        Second item is a list of predicate match objects.
        (This enables getting name info without doing another 
        re search on the names of the matching items.)
        If predicate is given, this will have equal length
        to the first item. (Otherwise it will be empty.)
        '''
        
        if (name is not None) and not isstringlike(name):
            raise TypeError("control name must be string-like")
        if (type is not None) and not isstringlike(type):
            raise TypeError("control type must be string-like")
        if (kind is not None) and not isstringlike(kind):
            raise TypeError("control kind must be string-like")
        if (id is not None) and not isstringlike(id):
            raise TypeError("control id must be string-like")
        if (label is not None) and not isstringlike(label):
            raise TypeError("control label must be string-like")
        if (predicate is not None) and not callable(predicate):
            raise TypeError("control predicate must be callable")

        found = None
        ambiguous = False

        results, p_results = [], []
        for control in self.controls:
            if name is not None and name != control.name:
                continue
            if type is not None and type != control.type:
                continue
            if kind is not None and not control.is_of_kind(kind):
                continue
            if id is not None and id != control.id:
                continue
            if predicate:
                p_result = predicate(control)
                if not predicate(control):
                    continue
            if label:
                for l in control.get_labels():
                    if l.text.find(label) > -1:
                        break
                else:
                    continue
            # append the control to the results list
            results.append(control)
            if predicate:
                p_results.append(p_result)

        return results, p_results

# XXX do the monkey-patch

def do_patch(origclass, patchclass):
    for key, value in patchclass.__dict__.iteritems():
        if key not in ('__doc__', '__module__'):
            setattr(origclass, key, value)

do_patch(ClientForm.HTMLForm, HTMLForm_patches)

def do_patch(origclass, patchclass):
    for key, value in patchclass.__dict__.iteritems():
        if key not in ('__doc__', '__module__'):
            setattr(origclass, key, value)
 

ExtendedHTMLForm = ClientForm.HTMLForm

# --
# Extended Browser
# --

# XXX Since HTMLForm is monkey-patched, we need no changes on
# the browser. However, code that wants to use the extended form
# class, should instantiate the extended browser class. This will
# enable us to get rid of monkey-patching later without the need
# to modify existing client code.

class ExtendedBrowser(mechanize.Browser):
    pass
